@@ -54,6 +54,10 @@ class TCPClient:
     def __repr__(self):
         return f'[{str(self.socket)},({self.__target_ip},{self.__target_port}),{self.__recv_buffer}]'
 
+    @property
+    def Address(self):
+        return (self.__target_ip, self.__target_port)
+
     def setup(self, target_ip: str, target_port: int = 25567, recv_buffer: int = 2048, on_connect: Callable = None,
               on_disconnect: Callable = None, on_receive: Callable = None):
         '''
@@ -154,7 +158,7 @@ class TCPClient:
                 if callable(self.on_disconnect):
                     self.on_disconnect()
 
-    def autorecv(self):
+    def autorecv(self) -> bool:
         '''
         function which activates the auto-receiving thread, automatically saving all incoming data
         '''
@@ -162,18 +166,25 @@ class TCPClient:
             self.__autorecv = True
             try:
                 self.__autorecv_thread.start()
+                return True
             except Exception as e:
                 self.event.exception.list.append((traceback.format_exc(), e))
                 self.event.exception.occurred = True
                 self.__autorecv_thread = threading.Thread(target=self.__reciving_automatic, daemon=True)
-                self.__autorecv_thread.start()
+                return False
+        elif self.__autorecv is True:
+            self.__autorecv = False
+            self.__autorecv_thread = threading.Thread(target=self.__reciving_automatic, daemon=True)
+            return True
+
+        return False
 
     def shutdown(self):
         '''
         sets the flag in the autorecv thread to False and closes the socket
         '''
         self.__autorecv = False
-        self.__autorecv_thread = threading.Thread(target=self.__reciving_automatic, daemon=True)
+        self.socket.shutdown(1)
         self.socket.close()
 
     def recv_data(self) -> bytes:
@@ -206,6 +217,19 @@ class TCPClient:
             self.event.exception.occurred = False
         return exceptions
 
+    def disconnect(self) -> bool:
+        '''
+        tries to disconnect from the Server
+        '''
+        try:
+            self.socket.shutdown(1)
+            self.socket.close()
+            return True
+        except Exception as e:
+            self.event.exception.occurred = True
+            self.event.exception.list.append((traceback.format_exc(), e))
+            return False
+
 
 class TCPServer:
     '''
@@ -261,6 +285,16 @@ class TCPServer:
     def __repr__(self):
         return f'[{str(self.socket)},({self.__IP},{self.__PORT}),{self.__recv_buffer}]'
 
+    def __perfom_disconnect(self, address):
+        self.clients.pop(address)
+        self.__allthreads.pop(address)
+
+        if callable(self.on_disconnect):
+            self.on_disconnect(address)
+
+        if self.max_connections is not None and len(self.clients) < self.max_connections:
+            self.run = True
+
     # Prepares the Server
     def setup(self, ip: str = socket.gethostname(), port: int = 25567, listen: int = 5, recv_buffer: int = 2048,
               handle_client=None, on_connect=None, on_disconnect=None, on_receive=None):
@@ -358,18 +392,13 @@ class TCPServer:
                     if callable(self.on_receive):
                         self.on_receive(client_socket, address, recved)
 
+        except ConnectionResetError as e:
+            pass
         except Exception as e:
             self.event.exception.list.append((traceback.format_exc(), e))
             self.event.exception.occurred = True
 
-            self.clients.pop(address)
-            self.__allthreads.pop(address)
-
-            if callable(self.on_disconnect):
-                self.on_disconnect(address)
-
-            if self.max_connections is not None and len(self.clients) < self.max_connections:
-                self.run = True
+            self.__perfom_disconnect(address)
 
     def __accept_clients(self):
         while self.__kill is False:
@@ -415,6 +444,7 @@ class TCPServer:
         elif self.__kill is False:
             raise Exception("Can't restart stopped or running thread")
 
+    @property
     def killed(self) -> bool:
         '''
         :return: returns if the accepting thread got killed
@@ -427,10 +457,15 @@ class TCPServer:
 
         :param address: the address of the client which you want to disconnect
         '''
-        client_socket = self.clients.get(address)[1]
-        self.clients.pop(address)
-        self.__allthreads.pop(address)
-        client_socket.close()
+        try:
+            client_socket = self.clients.get(address)[1]
+            self.clients.pop(address)
+            self.__allthreads.pop(address)
+            client_socket.shutdown(1)
+            client_socket.close()
+        except Exception as e:
+            self.event.exception.occurred = True
+            self.event.exception.list.append((traceback.format_exc(), e))
 
         if callable(self.on_disconnect):
             self.on_disconnect(address)
